package com.example.session.repository;

import com.hazelcast.map.EntryProcessor;
import com.hazelcast.map.ExtendedMapEntry;
import org.springframework.session.MapSession;

import java.time.Duration;
import java.time.Instant;
import java.util.Map;
import java.util.concurrent.TimeUnit;

public class CustomSessionUpdateEntryProcessor implements EntryProcessor<String, MapSession, Object> {

    private Instant lastAccessedTime;

    private Duration maxInactiveInterval;

    private Map<String, Object> delta;

    @Override
    public Object process(Map.Entry<String, MapSession> entry) {
        MapSession value = entry.getValue();
        if (value == null) {
            return Boolean.FALSE;
        }
        if (this.lastAccessedTime != null) {
            value.setLastAccessedTime(this.lastAccessedTime);
        }
        if (this.maxInactiveInterval != null) {
            value.setMaxInactiveInterval(this.maxInactiveInterval);
        }
        if (this.delta != null) {
            for (final Map.Entry<String, Object> attribute : this.delta.entrySet()) {
                if (attribute.getValue() != null) {
                    value.setAttribute(attribute.getKey(), attribute.getValue());
                }
                else {
                    value.removeAttribute(attribute.getKey());
                }
            }
        }
        if (this.maxInactiveInterval != null) {
            ((ExtendedMapEntry<String, MapSession>) entry).setValue(value, this.maxInactiveInterval.getSeconds(),
                    TimeUnit.SECONDS);
        }
        else {
            entry.setValue(value);
        }
        return Boolean.TRUE;
    }

    void setLastAccessedTime(Instant lastAccessedTime) {
        this.lastAccessedTime = lastAccessedTime;
    }

    void setMaxInactiveInterval(Duration maxInactiveInterval) {
        this.maxInactiveInterval = maxInactiveInterval;
    }

    void setDelta(Map<String, Object> delta) {
        this.delta = delta;
    }

}

